from typing import Dict, Iterable, Any
import time, json, os

import bottle

from logger import Logger
from servicestub import ServiceStub
from singleton import Singleton


class Crpt(metaclass=Singleton):
    log: Logger
    cases: Dict[str, Iterable[Dict[str, Any]]]

    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub: ServiceStub):
        self.log.info('Регистрация сервиса crpt')
        self.__updateCases()

        servicestub.route('/alpha/api/v4/true-api/cdn/info', method='GET', callback=self.cdnInfoA)
        servicestub.route('/bravo/api/v4/true-api/cdn/info', method='GET', callback=self.cdnInfoB)

        servicestub.route('/api/v4/true-api/codes/check', method='POST', callback=self.checkLegacy)
        servicestub.route('/alpha/api/v4/true-api/codes/check', method='POST', callback=self.checkA)
        servicestub.route('/bravo/api/v4/true-api/codes/check', method='POST', callback=self.checkB)
        servicestub.route('/charlie/api/v4/true-api/codes/check', method='POST', callback=self.checkC)

        servicestub.route('/alpha/api/v4/true-api/cdn/health/check', method='GET', callback=self.cdnHealthA)
        servicestub.route('/bravo/api/v4/true-api/cdn/health/check', method='GET', callback=self.cdnHealthB)
        servicestub.route('/charlie/api/v4/true-api/cdn/health/check', method='GET', callback=self.cdnHealthC)

    def checkCommon(self, prefix):
        self.__updateCases()

        self.log.info('Проверка маркировки')
        methodPath = prefix + '/api/v4/true-api/codes/check'

        return self.__getResult(methodPath)

    def checkA(self):
        return self.checkCommon("/alpha")

    def checkB(self):
        return self.checkCommon("/bravo")

    def checkC(self):
        return self.checkCommon("/charlie")

    def checkLegacy(self):
        return self.checkCommon("")

    def cdnInfoCommon(self, prefix):
        self.__updateCases()

        self.log.info('Получение списка CDN')
        methodPath = prefix + '/api/v4/true-api/cdn/info'
        return self.__getResult(methodPath)

    def cdnInfoA(self):
        return self.cdnInfoCommon("/alpha")

    def cdnInfoB(self):
        return self.cdnInfoCommon("/bravo")

    def cdnHealthCommon(self, prefix):
        self.__updateCases()
        self.log.info('Получение статуса CDN')

        methodPath = prefix + '/api/v4/true-api/cdn/health/check'
        return self.__getResult(methodPath)

    def cdnHealth(self):
        return self.cdnHealthCommon('')

    def cdnHealthA(self):
        return self.cdnHealthCommon('/alpha')

    def cdnHealthB(self):
        return self.cdnHealthCommon('/bravo')

    def cdnHealthC(self):
        return self.cdnHealthCommon('/charlie')

    def __getResult(self, methodPath: str):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        time.sleep(self.cases[methodPath].get('timeout', 0) / 1000)

        responseSuffix = self.cases[methodPath].get('responseSuffix', '')
        body = json.dumps(self.cases[methodPath].get('response' + responseSuffix))
        response = bottle.HTTPResponse(status=self.cases[methodPath].get('status', 200), body=body)
        response.add_header("Content-Type", "application/json")
        return response

    def __updateCases(self):
        self.cases.clear()
        try:
            for file in ["cdninfo", "cdnhealth", "codescheck"]:
                with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), file + '.json'), 'r') as cases:
                    self.cases.update(json.load(cases))
        except Exception as e:
            self.log.error(str(e))


instance = Crpt()
